<div role="search">
  <form id ="rtd-search-form" class="wy-form" action="{{ base_url }}/search.html" method="get">
      <input type="text" name="q" placeholder="{% trans %}Search docs{% endtrans %}" title="{% trans %}Type search term here{% endtrans %}" />
  </form>
</div>
