{% extends "main.html" %}

{% block content %}

  <h1 id="search">{% trans %}Search Results{% endtrans %}</h1>

  <form id="content_search" action="search.html">
    <span role="status" aria-live="polite" class="ui-helper-hidden-accessible"></span>
    <input name="q" id="mkdocs-search-query" type="text" class="search_input search-query ui-autocomplete-input" placeholder="{% trans %}Search the Docs{% endtrans %}" autocomplete="off" autofocus title="{% trans %}Type search term here{% endtrans %}">
  </form>

  <div id="mkdocs-search-results" class="search-results" data-no-results-text="{% trans %}No results found{% endtrans %}">
    {% trans %}Searching...{% endtrans %}
  </div>

{% endblock %}
